# -*- coding: utf-8 -*-
"""
properties module: message properties/attributes definitions.
"""
from .base import BaseProperty
from .base import NormalProperty
from .base import ChoiceProperty
from .string_property import StringProperty
from .boolean_property import BooleanProperty
from .float_property import FloatProperty
from .integer_property import IntegerProperty
from .date_property import DateProperty
from .datetime_property import DateTimeProperty
from .object_property import ObjectProperty
from .array_property import ArrayProperty
from .dict_property import DictProperty
from .file_property import FileProperty
