# -*- coding: utf-8 -*-
"""
Module exceptions.py
-----------------------------------
Mercury error exceptions classes.
"""


class ResourceSpecError(Exception):
    """
    Exception class representing errors of a mercury resource specification/implementation.
    """
    pass
